@@ -2,14 +2,17 @@ use strict;
 use warnings;
 
 package WebService::Geocodio::Location;
-$WebService::Geocodio::Location::VERSION = '0.02';
-use Moo;
+{
+  $WebService::Geocodio::Location::VERSION = '0.03';
+}
+
+use Moo::Lax;
 use Carp qw(confess);
 
 # ABSTRACT: Location object for use with Geocod.io service.
 
 
-has [qw(number street suffix city state zip formatted lat lng accuracy)] => (
+has [qw(number street suffix postdirection city state zip formatted lat lng accuracy fields)] => (
     is => 'ro',
     predicate => 1,
 );
@@ -23,7 +26,8 @@ sub BUILDARGS {
         my $hr = $args[0];
         $out->{accuracy} = $hr->{accuracy} if exists $hr->{accuracy};
         $out->{formatted} = $hr->{formatted_address} if exists $hr->{formatted_address};
-        map { $out->{$_} = $hr->{address_components}->{$_} if exists $hr->{address_components}->{$_} } qw(number street suffix city state zip);
+        $out->{fields} = $hr->{fields} if exists $hr->{fields};
+        map { $out->{$_} = $hr->{address_components}->{$_} if exists $hr->{address_components}->{$_} } qw(number street suffix postdirection city state zip);
         map { $out->{$_} = $hr->{location}->{$_} if exists $hr->{location}->{$_} } qw(lat lng);
     }
     elsif ( @args % 2 == 1 ) {
@@ -36,18 +40,26 @@ sub BUILDARGS {
     return $out;
 }
 
-sub TO_JSON {
+sub _forward_formatting {
     my $self = shift;
 
     return $self->formatted if $self->has_formatted;
 
     if ( ( not $self->has_zip ) && ( not ( $self->has_city && $self->has_state ) ) ) {
-        confess "A zip or city, state pair is required.\n";
+        confess "A zip or city-state pair is required.\n";
     }
 
     my $s;
     if ( $self->has_number && $self->has_street && $self->suffix ) {
-        $s .= join " ", (map { $self->$_ } qw(number street suffix));
+        my @f;
+        if ( $self->has_postdirection ) {
+            @f = qw(number postdirection street suffix);
+        }
+        else {
+            @f = qw(number street suffix);
+        }
+
+        $s .= join " ", (map {; $self->$_ } @f);
         $s .= ", ";
     }
 
@@ -55,12 +67,23 @@ sub TO_JSON {
         $s .= $self->zip
     }
     else {
-        $s .= join ", ", (map { $self->$_ } qw(city state));
+        $s .= join ", ", (map {; $self->$_ } qw(city state));
     }
 
     return $s;
 }
 
+sub _reverse_formatting {
+    my $self = shift;
+
+    if ( not ( $self->has_lat && $self->has_lng ) ) {
+        confess "lat-lng pair is required\n";
+    }
+
+    return join ",", ( map {; $self->$_ } qw(lat lng) );
+}
+
+
 1;
 
 __END__
@@ -73,7 +96,7 @@ WebService::Geocodio::Location - Location object for use with Geocod.io service.
 
 =head1 VERSION
 
-version 0.02
+version 0.03
 
 =head1 ATTRIBUTES
 
@@ -85,6 +108,10 @@ Buiding number
 
 A street name identifier
 
+=head2 postdirection
+
+A address direction like 'SW' or 'S'
+
 =head2 suffix
 
 The type of street 'Ave', 'St', etc.
@@ -119,12 +146,20 @@ A float from 0 -> 1 representing the confidence of the lookup results
 
 The full address as formatted by the service.
 
+=head2 fields
+
+A hashref to any fields data requested. Will most likely be replaced by field specific
+classes in the near future. 
+
+For now, the field name and its subkeys are all naked hash keys. (Sorry. This will
+improve.)
+
 =head1 METHODS
 
 =head2 new
 
-The constructor accepts either a bare string OR a list of key/value pairs where the keys are
-the attribute names.
+The constructor accepts either a bare string OR a list of key/value pairs where
+the keys are the attribute names.
 
 =head1 AUTHOR
 
